@@ -1,4 +1,6 @@
 import argparse
+import os
+from pathlib import Path
 import pytest
 import sys
 from tuxmake.build import BuildInfo
@@ -26,13 +28,13 @@ def args(called):
 def test_basic_build(builder, monkeypatch):
     monkeypatch.setattr(sys, "argv", ["tuxmake"])
     tuxmake()
-    assert builder.call_args[1] == {"tree": "."}
+    assert args(builder).tree == "."
 
 
 def test_basic_build_with_directory(linux, builder):
     tree = str(linux)
     tuxmake("--directory", tree)
-    assert builder.call_args[1] == {"tree": tree}
+    assert args(builder).tree == tree
 
 
 def test_build_from_sys_argv(monkeypatch, builder):
@@ -57,6 +59,24 @@ class TestTargets:
         assert args(builder).targets == ["config", "kernel"]
 
 
+class TestMakeVariables:
+    def test_basic(self, builder):
+        tuxmake("FOO=BAR")
+        assert args(builder).make_variables == {"FOO": "BAR"}
+        assert "targets" not in args(builder).__dict__
+
+    def test_make_vars_and_targets(self, builder):
+        tuxmake("FOO=BAR", "config")
+        assert args(builder).make_variables == {"FOO": "BAR"}
+        assert args(builder).targets == ["config"]
+
+    def test_rejects_multiple_equal_signs(self, builder):
+        with pytest.raises(SystemExit) as exit:
+            tuxmake("FOO=BAR=QUX", "config")
+        builder.assert_not_called()
+        assert exit.value.code == 1
+
+
 class TestKConfig:
     def test_kconfig(self, builder):
         tuxmake("--kconfig=olddefconfig")
@@ -97,6 +117,35 @@ class TestRuntime:
         assert args(builder).runtime == "docker"
 
 
+class TestImage:
+    @pytest.fixture(autouse=True)
+    def environment(self, monkeypatch):
+        env = {}
+        monkeypatch.setattr(os, "environ", env)
+        return env
+
+    def test_implies_runtime_docker(self, builder):
+        tuxmake("--image=foobar")
+        assert args(builder).runtime == "docker"
+
+    def test_does_not_override_explicit_runtime(self, builder):
+        tuxmake("--image=foobar", "--runtime=podman")
+        assert args(builder).runtime == "podman"
+
+    def test_not_passed_to_builder_class(self, builder):
+        tuxmake("--image=foobar")
+        assert "image" not in builder.call_args[1].keys()
+
+    def test_sets_TUXMAKE_IMAGE(self, environment):
+        tuxmake("--image=foobar")
+        assert environment["TUXMAKE_IMAGE"] == "foobar"
+
+    def test_backwards_compat_with_docker_image(self, environment, builder):
+        tuxmake("--docker-image=foobar")
+        assert environment["TUXMAKE_IMAGE"] == "foobar"
+        assert "docker_image" not in builder.call_args[1].keys()
+
+
 class TestVerbosity:
     def test_verbose(self, builder):
         tuxmake("--verbose")
@@ -188,6 +237,13 @@ class TestList:
         assert "clang" in out
         assert "clang-10" in out
 
+    def test_list_runtimes(self, builder, capsys):
+        tuxmake("--list-runtimes")
+        builder.assert_not_called()
+        out, _ = capsys.readouterr()
+        assert "null" in out
+        assert "docker" in out
+
 
 class TestPrintSupportMatrix:
     def test_print_support_matrix(self, builder, capsys):
@@ -208,3 +264,50 @@ class TestPrintSupportMatrix:
         builder.assert_not_called()
         out, _ = capsys.readouterr()
         assert "\033" in out
+
+
+class TestDebug:
+    def test_shell(self, builder, mocker):
+        run_cmd = builder.return_value.run_cmd
+        tuxmake("--shell")
+        run_cmd.assert_called_with(["bash"], interactive=True)
+
+    def test_debug(self, builder, mocker):
+        tuxmake("--debug")
+        assert args(builder).debug
+
+
+class TestOutputDir:
+    def test_output_dir(self, builder):
+        tuxmake("--output-dir=/path/to/output")
+        assert args(builder).output_dir == Path("/path/to/output")
+
+    def test_output_dir_relative_to_absolute(self, builder):
+        cwd = Path.cwd()
+        tuxmake("--output-dir=output")
+        assert args(builder).output_dir == (cwd / "output")
+
+
+class TestBuildDir:
+    def test_build_dir(self, builder):
+        tuxmake("--build-dir=/path/to/build")
+        assert args(builder).build_dir == Path("/path/to/build")
+
+
+class TestOptionsFromEnvironment:
+    def test_options_from_environment(self, builder, monkeypatch):
+        monkeypatch.setenv("TUXMAKE", "--debug --verbose")
+        tuxmake("config")
+        assert args(builder).debug
+        assert args(builder).verbose
+
+    def test_command_line_overrides_environment(self, builder, monkeypatch):
+        monkeypatch.setenv("TUXMAKE", "--toolchain=gcc")
+        tuxmake("--toolchain=clang")
+        assert args(builder).toolchain == "clang"
+
+    def test_command_line_plus_sys_argv(self, builder, monkeypatch):
+        monkeypatch.setenv("TUXMAKE", "--toolchain=gcc")
+        monkeypatch.setattr(sys, "argv", ["tuxmake", "--toolchain=clang"])
+        tuxmake()
+        assert args(builder).toolchain == "clang"
