@@ -4,9 +4,11 @@ import subprocess
 import shutil
 import pathlib
 import pytest
+from tuxmake import __version__
 from tuxmake.build import Build
 from tuxmake.exceptions import UnsupportedMetadataType
 from tuxmake.metadata import Metadata
+from tuxmake.metadata import MetadataExtractor
 
 
 @pytest.fixture(autouse=True, scope="module")
@@ -40,6 +42,16 @@ def build(linux):
     return b
 
 
+class TestExtractor:
+    @pytest.fixture
+    def extractor(self, build):
+        return MetadataExtractor(build)
+
+    @pytest.mark.parametrize("data", [None, "", "{}"])
+    def test_dont_crash_on_corner_cases(self, extractor, data):
+        extractor.read_json(data)
+
+
 class TestMetadata:
     def test_invalid_type(self, mocker):
         m = Metadata("source")
@@ -47,6 +59,12 @@ class TestMetadata:
         with pytest.raises(UnsupportedMetadataType):
             m.__init_config__()
 
+    def test_order_all(self):
+        cls = Metadata.all()
+        source = next(c for c in cls if c.name == "source")
+        git = next(c for c in cls if c.name == "git")
+        assert cls.index(source) < cls.index(git)
+
 
 class TestKernelVersion:
     def test_happy_path(self, build):
@@ -101,3 +119,21 @@ class TestArtifacts:
 
     def test_dtbs(self, build):
         assert type(build.metadata["artifacts"]["dtbs"]) is list
+
+
+class TestVmlinux:
+    @pytest.mark.parametrize(
+        "field", ["bss_size", "data_size", "text_size", "file_size"]
+    )
+    def test_field(self, build, field):
+        assert type(build.metadata["vmlinux"][field]) is int
+
+
+class TestSystemMap:
+    def test_text_offset(self, build):
+        assert build.metadata["system_map"]["text_offset"] == "0xc1000000"
+
+
+class TestTuxmake:
+    def test_tuxmake_versino(self, build):
+        assert build.metadata["tuxmake"]["version"] == __version__
