@@ -43,6 +43,12 @@ class TestConfig:
     def test_artifacts(self, config):
         assert config.artifacts["config"] == ".config"
 
+    def test_does_nothing_if_dot_config_already_exists(self, config, build):
+        build.kconfig = "defconfig"
+        (build.build_dir / ".config").touch()
+        config.prepare()
+        assert config.commands == []
+
 
 class TestDebugKernel:
     def test_commands(self, build):
@@ -60,6 +66,22 @@ class TestKernel:
         assert kernel.dependencies == ["config"]
 
 
+class TestModules:
+    @pytest.fixture
+    def modules(self, build):
+        return Target("modules", build)
+
+    def test_install_modules(self, modules):
+        assert modules.commands[0][0:2] == ["{make}", "modules_install"]
+
+    def test_strip_modules(self, modules):
+        assert "INSTALL_MOD_STRIP=1" in modules.commands[0]
+
+    def test_depends_on_kernel_and_debugkernel(self, modules):
+        assert "debugkernel" in modules.dependencies
+        assert "kernel" in modules.dependencies
+
+
 class TestDtbs:
     def test_commands(self, build):
         dtbs = Target("dtbs", build)
