@@ -1,9 +1,8 @@
 from pathlib import Path
 import re
-import shlex
 import urllib.request
 
-from tuxmake.config import ConfigurableObject
+from tuxmake.config import ConfigurableObject, split_commands
 from tuxmake.exceptions import InvalidKConfig
 from tuxmake.exceptions import UnsupportedTarget
 from tuxmake.exceptions import UnsupportedKconfig
@@ -48,15 +47,7 @@ class Target(ConfigurableObject):
 
     def __split_cmds__(self, section, item):
         s = self.config[section].get(item)
-        if not s:
-            return []
-        result = [[]]
-        for item in shlex.split(s):
-            if item == "&&":
-                result.append([])
-            else:
-                result[-1].append(item)
-        return result
+        return split_commands(s)
 
     def prepare(self):
         pass
@@ -67,11 +58,15 @@ class Config(Target):
         super().__init_config__()
 
     def prepare(self):
+        olddefconfig = False
         build_dir = self.build.build_dir
         config = build_dir / ".config"
         conf = self.build.kconfig
+        if config.exists():
+            return
         if self.handle_url(config, conf) or self.handle_local_file(config, conf):
             self.build.log(f"# {conf} -> {config}")
+            olddefconfig = True
         elif self.handle_make_target(conf):
             pass
         else:
@@ -107,6 +102,8 @@ class Config(Target):
                     *merge,
                 ]
             )
+            olddefconfig = True
+        if olddefconfig:
             self.commands.append(["{make}", "olddefconfig"])
 
     def handle_url(self, config, url):
@@ -131,14 +128,14 @@ class Config(Target):
         return True
 
     def handle_make_target(self, t):
-        if re.match(r"^\w+config$", t):
+        if re.match(r"^[\w\-]+config$", t):
             self.commands.append(["{make}", t])
             return True
         else:
             return False
 
     def handle_in_tree_config(self, t):
-        if re.match(r"^\w+\.config$", t):
+        if re.match(r"^[\w\-]+.config$", t):
             self.commands.append(["{make}", t])
             return True
         else:
