@@ -1,6 +1,7 @@
 import os
+import subprocess
 from pathlib import Path
-from tuxmake.config import ConfigurableObject
+from tuxmake.config import ConfigurableObject, split_commands
 from tuxmake.exceptions import UnsupportedWrapper
 
 
@@ -26,14 +27,19 @@ class Wrapper(ConfigurableObject):
         self.environment = {
             k: expand(k, v) for k, v in self.config["environment"].items()
         }
+        self.prepare_cmds = split_commands(self.config["commands"]["prepare"])
 
-    def prepare(self):
+    def prepare(self, build):
         for k, v in self.environment.items():
             if k.endswith("_DIR"):
                 Path(v).mkdir(parents=True, exist_ok=True)
+        for cmd in self.prepare_cmds:
+            build.run_cmd(cmd, stdout=subprocess.DEVNULL)
 
     def wrap(self, makevars):
         cross = makevars.get("CROSS_COMPILE", "")
+        if makevars.get("LLVM") == "1":
+            return {"CC": f"{self.name} clang", "HOSTCC": f"{self.name} clang"}
         return {
             k: f"{self.name} {v}" for k, v in makevars.items() if k in ("CC", "HOSTCC")
         } or {"CC": f"{self.name} {cross}gcc", "HOSTCC": f"{self.name} gcc"}
@@ -44,7 +50,7 @@ class NoWrapper(Wrapper):
         self.environment = {}
         self.name = "none"
 
-    def prepare(self):
+    def prepare(self, build):
         pass
 
     def wrap(self, makevars):
